Public Class frmMain

   Private ReadOnly OptionFile As String = My.Application.Info.DirectoryPath & "\Options.xml"

   Private ReadThread As Threading.Thread
    Private COM As System.IO.Ports.SerialPort
   Private Writer As IO.StreamWriter

   Private negX As Byte = 128
   Private stepX() As Byte = {32, 64}
   Private negY As Byte = 8
   Private stepY() As Byte = {2, 4}

   Friend Options As ProgOptions
   
   Private MouseEnabled As Boolean
   Private ReadingInfos As Boolean


   Friend dLeft As Boolean
   Friend dRight As Boolean
   Friend dUp As Boolean
   Friend dDown As Boolean
   Friend dSpeedX As Integer = 0
   Friend dSpeedY As Integer = 0





   Public Declare Sub mouse_event Lib "user32" (ByVal dwFlags As Integer, ByVal dx As Integer, ByVal dy As Integer, ByVal cButtons As Integer, ByVal dwExtraInfo As Integer)
   Const LEFTDOWN As Integer = &H2
   Const LEFTUP As Integer = &H4
   Const MIDDLEDOWN As Integer = &H20
   Const MIDDLEUP As Integer = &H40
   Const RIGHTDOWN As Integer = &H8
   Const RIGHTUP As Integer = &H10













   ''' <summary>
   ''' ffnet den ComPort fr die bertragung und stellt ber die
   ''' Writer-Variable einen StreamWriter zum Senden  von Befehlen
   ''' an den Knoten zur Verfgung. Ankommende Daten werden ber die
   ''' Read-Methode in einem eigenen Thread abgefangen
   ''' </summary>
   ''' <remarks></remarks>
   Private Sub OpenComPort()
      If COM.IsOpen Then Exit Sub

      'Com-Port einstellen
      COM.BaudRate = 115200
      COM.DataBits = 8
      COM.Parity = IO.Ports.Parity.None
      COM.StopBits = IO.Ports.StopBits.One
      COM.NewLine = "\r\n"

      Try
         COM.Open()
      Catch ex As Exception
         MsgBox(ex.Message, MsgBoxStyle.Exclamation, "Fehler beim ffnen des COM-Ports")
         COM = Nothing
         End
         Exit Sub
      End Try

      Log("COM3 geffnet...")

      If Writer IsNot Nothing Then
         Writer.Close()
      End If

      'WriterSream erstellen, ber den Befehle an den Knoten gesendet werden
      Writer = New IO.StreamWriter(COM.BaseStream)
      Console.WriteLine("Writer - Stream aufgebaut")

      If ReadThread IsNot Nothing Then
         ReadThread.Abort()
      End If

      ReadThread = New Threading.Thread(AddressOf Read)
      ReadThread.IsBackground = True
      ReadThread.Start()
      Console.WriteLine("ReadThread wurde gestartet")

   End Sub




   Private Sub ReadNodeInfos()
      'ReadingInfos = True

      ReadThread.Suspend()

      Dim R As New IO.StreamReader(COM.BaseStream)


      'Watch stoppen, damit keine weiteren Daten ankommen
      'Writer.WriteLine("StopWatch")
      'Writer.Flush()
      'Log("StopWatch geschickt...")

      'Dim L As String = R.ReadLine

      Writer.WriteLine("ShowInfos")
      Writer.Flush()
      Log("ShowInfos geschickt...")



      Dim L As String = R.ReadLine
      L = R.ReadLine
      Dim P As Integer = 0

      'Calibrated
      If Not L.StartsWith("Calibrated:") Then
         MsgBox("Fehler beim Parsen der Infos", MsgBoxStyle.Exclamation, "Fehler")
         Exit Sub
      End If

      'Eventuelle Kallibrierung
      If L.Trim.EndsWith("0") Then
         If MsgBox("Der Sensorknoten wurde nicht kallibriert. Mchten Sie die Kallibrierung jetzt durchfhren?", _
          MsgBoxStyle.YesNo Or MsgBoxStyle.Question, "Keine Kallibrierung") = MsgBoxResult.No Then
            Me.Close()
            Exit Sub
         End If

         Calibrate()
         'L = R.ReadLine 'Callibrated-Besttigung berlesen
      End If

      'Started
      L = R.ReadLine
      If Not L.StartsWith("Started:") Then
         MsgBox("Fehler beim Parsen der Infos", MsgBoxStyle.Exclamation, "Fehler")
         Exit Sub
      End If

      'Eventuelle mit Standardwerten starten
      If L.Trim.EndsWith("0") Then
         R.DiscardBufferedData()
         SendStartWatch()
         L = R.ReadLine
         ReadThread.Resume()
         Exit Sub
      End If


      'RefreshRate einstellen
      L = R.ReadLine
      If Not L.StartsWith("WatchRate:") Then
         MsgBox("Fehler beim Parsen der Infos", MsgBoxStyle.Exclamation, "Fehler")
         Exit Sub
      End If
      If Integer.TryParse(L.Substring(10), P) Then
         RefreshRate.Value = RefreshRate.Maximum - P
      End If


      '4 Zeilen berspringen (Base) und SensitySoft lesen
      For I As Integer = 1 To 5
         L = R.ReadLine
      Next

      If Not L.StartsWith("SensitySoft:") Then
         MsgBox("Fehler beim Parsen der Infos", MsgBoxStyle.Exclamation, "Fehler")
         Exit Sub
      End If
      L = R.ReadLine()
      'X
      If Integer.TryParse(L.Substring(2), P) Then
         SensityX.Value = SensityX.Maximum - P
      End If
      L = R.ReadLine
      'Y
      If Integer.TryParse(L.Substring(2), P) Then
         SensityY.Value = SensityY.Maximum - P
      End If
      R.ReadLine() 'Z - Achse interessiert nicht

      'MouseStepRange
      L = R.ReadLine()
      If Not L.StartsWith("MouseStepRange:") Then
         MsgBox("Fehler beim Parsen der Infos", MsgBoxStyle.Exclamation, "Fehler")
         Exit Sub
      End If
      R.ReadLine() 'der 0. ist immer 0

      L = R.ReadLine()
      If Integer.TryParse(L.Substring(5), P) Then
         StepRange0.SplitterDistance = P
      End If

      L = R.ReadLine()
      If Integer.TryParse(L.Substring(5), P) Then
         StepRange1.SplitterDistance = P - StepRange0.SplitterDistance
      End If

      R.DiscardBufferedData()

      ReadThread.Resume()



   End Sub

   ''' <summary>
   ''' Sendet den Befehl an den Knoten, der ihn veranlasst auf Bewegungen zu reagieren.
   ''' </summary>
   ''' <remarks></remarks>
   Private Sub SendStartWatch()
      Writer.WriteLine("StartWatch " & (SensityX.Maximum - SensityX.Value).ToString & " " & _
                                       (SensityY.Maximum - SensityY.Value).ToString & " " & _
                                       "-1 " & _
                                       (RefreshRate.Maximum - RefreshRate.Value).ToString & " " & _
                                       StepRange0.SplitterDistance.ToString & " " & _
                                       (StepRange0.SplitterDistance + StepRange1.SplitterDistance).ToString)
      Writer.Flush()
      Log("StartWatch wurde gesendet...")
   End Sub

   ''' <summary>
   ''' Behandelt ankommende Daten vom Knoten
   ''' </summary>
   ''' <remarks></remarks>
   Private Sub Read()
      Do

         Dim B(0) As Byte
         Try
            Dim BR As Integer = COM.BaseStream.Read(B, 0, 1)
            If Not ReadingInfos Then


               If BR < B.Length Then
                  Log("Block nicht vollstndig erhalten")
               Else
                  'Log("Paket erhalten " & Now)
                  Log(Convert.ToString(B(0), 2))

                  If Options.InvButtons Then
                     'Maustaste des Buttons2
                     If CBool(B(0) And 16) Then
                        'einfachen Klick simulieren
                        mouse_event(RIGHTDOWN, 0, 0, 0, 0) ' Rechts-Maus-Klick RUNTER
                        mouse_event(RIGHTUP, 0, 0, 0, 0) ' Rechts-Maus-Klick RAUF 
                     End If
                     'Maustaste des Buttons1
                     If CBool(B(0) And 1) Then
                        'einfachen Klick simulieren
                        mouse_event(LEFTDOWN, 0, 0, 0, 0) ' Links-Maus-Klick RUNTER
                        mouse_event(LEFTUP, 0, 0, 0, 0) ' Links-Maus-Klick RAUF 
                     End If
                  Else
                     'Maustaste des Buttons1
                     If CBool(B(0) And 16) Then
                        'einfachen Klick simulieren
                        mouse_event(LEFTDOWN, 0, 0, 0, 0) ' Links-Maus-Klick RUNTER
                        mouse_event(LEFTUP, 0, 0, 0, 0) ' Links-Maus-Klick RAUF 
                     End If
                     'Maustaste des Buttons2
                     If CBool(B(0) And 1) Then
                        'einfachen Klick simulieren
                        mouse_event(RIGHTDOWN, 0, 0, 0, 0) ' Rechts-Maus-Klick RUNTER
                        mouse_event(RIGHTUP, 0, 0, 0, 0) ' Rechts-Maus-Klick RAUF 
                     End If
                  End If


                  'X-Koordinate
                  If CBool(B(0) And negX) Then
                     'Rechtsbewegung
                     If CBool(B(0) And stepX(0)) And CBool(B(0) And stepX(1)) Then
                        Log("Rechts Stufe 3")
                        dSpeedX = 3
                        dRight = True
                        dLeft = False
                        MoveMouse(Options.InvX * Options.SpeedX(2))

                     ElseIf CBool(B(0) And stepX(0)) Then
                        Log("Rechts Stufe 1")
                        dSpeedX = 1
                        dRight = True
                        dLeft = False
                        MoveMouse(Options.InvX * Options.SpeedX(0))
                     ElseIf CBool(B(0) And stepX(1)) Then
                        Log("Rechts Stufe 2")
                        dSpeedX = 2
                        dRight = True
                        dLeft = False
                        MoveMouse(Options.InvX * Options.SpeedX(1))
                     End If
                  Else
                     'Linksbewegung
                     If CBool(B(0) And stepX(0)) And CBool(B(0) And stepX(1)) Then
                        Log("Links Stufe 3")
                        dSpeedX = 3
                        dRight = False
                        dLeft = True
                        MoveMouse(-Options.InvX * Options.SpeedX(2))
                     ElseIf CBool(B(0) And stepX(1)) Then
                        Log("Links Stufe 2")
                        dSpeedX = 2
                        dRight = False
                        dLeft = True
                        MoveMouse(-Options.InvX * Options.SpeedX(1))
                     ElseIf CBool(B(0) And stepX(0)) Then
                        Log("Links Stufe 1")
                        dSpeedX = 1
                        dRight = False
                        dLeft = True
                        MoveMouse(-Options.InvX * Options.SpeedX(0))
                     End If
                  End If

                  'Y-Koordinate
                  If CBool(B(0) And negY) Then
                     'Runter
                     If CBool(B(0) And stepY(0)) And CBool(B(0) And stepY(1)) Then
                        Log("Hoch Stufe 3")
                        dSpeedY = 3
                        dUp = True
                        dDown = False
                        MoveMouse(, -Options.InvY * Options.SpeedY(2))
                     ElseIf CBool(B(0) And stepY(1)) Then
                        Log("Hoch Stufe 2")
                        dSpeedY = 2
                        dUp = True
                        dDown = False
                        MoveMouse(, -Options.InvY * Options.SpeedY(1))
                     ElseIf CBool(B(0) And stepY(0)) Then
                        Log("Hoch Stufe 1")
                        dSpeedY = 1
                        dUp = True
                        dDown = False
                        MoveMouse(, -Options.InvY * Options.SpeedY(0))
                     End If
                  Else
                     'Hoch
                     If CBool(B(0) And stepY(0)) And CBool(B(0) And stepY(1)) Then
                        Log("Runter Stufe 3")
                        dSpeedY = 3
                        dUp = False
                        dDown = True
                        MoveMouse(, Options.InvY * Options.SpeedY(2))
                     ElseIf CBool(B(0) And stepY(1)) Then
                        Log("Runter Stufe 2")
                        dSpeedY = 2
                        dUp = False
                        dDown = True
                        MoveMouse(, Options.InvY * Options.SpeedY(1))
                     ElseIf CBool(B(0) And stepY(0)) Then
                        Log("Runter Stufe 1")
                        dSpeedY = 1
                        dUp = False
                        dDown = True
                        MoveMouse(, Options.InvY * Options.SpeedY(0))
                     End If
                  End If
               End If



            End If
         Catch ex As Threading.ThreadAbortException
            Log("ReadThread wurde beendet")
            Exit Do
         Catch e As Exception
            Log("Unbekannter Fehler: " & e.Message)
            Exit Do
         End Try





      Loop
   End Sub

   ''' <summary>
   ''' Bewegt den Mauszeiger zur angegebenen Koordinate
   ''' </summary>
   ''' <param name="X"></param>
   ''' <param name="Y"></param>
   ''' <remarks></remarks>
   Private Sub MoveMouse(Optional ByVal X As Integer = 0, Optional ByVal Y As Integer = 0)
      Try
         Cursor.Position = New Point(Cursor.Position.X + X, Cursor.Position.Y + Y)
      Catch ex As Exception

      End Try
   End Sub

   Private Sub Calibrate(Optional ByVal AutoStart As Boolean = False)
      'Watch stoppen, damit keine weiteren Daten ankommen
      Writer.WriteLine("StopWatch")
      Writer.Flush()
      Log("StopWatch geschickt...")

      MsgBox("Halten Sie den Knoten in der Ausgangsposition und drcken Sie auf OK, um den Knoten fr diese Ausgansposition zu kallibrieren!", _
         MsgBoxStyle.Information, "Kallibrierung")

      Writer.WriteLine("Calibrate 1")
      Writer.Flush()
      Log("Calibrate 1 geschickt...")

      MsgBox("Kallibrierung erfolgreich ausgefhrt", MsgBoxStyle.Information, "Info")

      If AutoStart Then SendStartWatch()
   End Sub

   Private Sub HideMe()
      If frmInfo.Visible Then frmInfo.Close()
      Me.Hide()
      Me.ShowInTaskbar = False
      If Noty.Tag Is Nothing Then
         Noty.ShowBalloonTip(4000)
         Noty.Tag = New Object
      End If
   End Sub

   Private Sub UseOptions()
      If Options Is Nothing Then Exit Sub
      With Options
         chkInvX.Checked = Options.InvX = -1
         chkInvY.Checked = Options.InvY = -1

         Speed1X.Value = Options.SpeedX(0)
         Speed2X.Value = Options.SpeedX(1)
         Speed3X.Value = Options.SpeedX(2)
         Speed1Y.Value = Options.SpeedY(0)
         Speed2Y.Value = Options.SpeedY(1)
         Speed3Y.Value = Options.SpeedY(2)

         chkInvButtons.Checked = Options.InvButtons
      End With
   End Sub

   Private Sub OvertakeOptions()
      If Options Is Nothing Then Exit Sub

      If chkInvX.Checked Then
         Options.InvX = -1
      Else
         Options.InvX = 1
      End If

      If chkInvY.Checked Then
         Options.InvY = -1
      Else
         Options.InvY = 1
      End If

      Options.SpeedX(0) = Speed1X.Value
      Options.SpeedX(1) = Speed2X.Value
      Options.SpeedX(2) = Speed3X.Value

      Options.SpeedY(0) = Speed1Y.Value
      Options.SpeedY(1) = Speed2Y.Value
      Options.SpeedY(2) = Speed3Y.Value

      Options.InvButtons = chkInvButtons.Checked

      Options.Save(OptionFile)
   End Sub










   ''' <summary>
   ''' Diese Methode kann spter umfunktioniert werden, 
   ''' so dass direkt in eine Datei geschrieben wird
   ''' </summary>
   ''' <param name="Msg"></param>
   ''' <remarks></remarks>
   Private Sub Log(ByVal Msg As String)
      Console.WriteLine(Msg)
   End Sub

   Private Sub Log()
      Log("")
   End Sub



#Region "Controls EventHandler"

   Private Sub frmMain_Shown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shown
      If My.Application.CommandLineArgs.Count > 0 Then
         If My.Application.CommandLineArgs.Contains("-silent") Then

            HideMe()
         End If
      End If
   End Sub

   Private Sub frmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
      
        Options = ProgOptions.LoadFromFile(OptionFile)
      If Options Is Nothing Then Options = New ProgOptions 'Wenn keine OptionFile existiert, Standardwerte laden
      UseOptions()
      OpenComPort()
      ReadNodeInfos()

   End Sub

   Private Sub frmMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
      e.Cancel = True
      HideMe()
   End Sub

   Private Sub btnOvertake_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOvertake.Click

      'Optionen bernehmen und speichern
      OvertakeOptions()

      'Watch stoppen, damit keine weiteren Daten ankommen
      Writer.WriteLine("StopWatch")
      Writer.Flush()
      Log("StopWatch geschickt...")

      Threading.Thread.Sleep(500) '500 Millisekunden wareten, bevor Timer wieder gestartet wird

      'Bewegungs-berwachungsroutine des Knotens starten
      SendStartWatch()
   End Sub


   Private Sub btnCalibrate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCalibrate.Click
      Calibrate()
   End Sub

   Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOK.Click
      btnOvertake_Click(Nothing, Nothing)

      'In Tray minimieren
      HideMe()
   End Sub

   Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
      HideMe()
   End Sub

   Private Sub AnzeigenToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AnzeigenToolStripMenuItem.Click
      Me.Show()
      Me.ShowInTaskbar = True
   End Sub

   Private Sub BeendenToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BeendenToolStripMenuItem.Click
      If MsgBox("Soll das Programm wirklich geschlossen werden", MsgBoxStyle.YesNo Or MsgBoxStyle.Question, "Programm schlieen?") _
       = MsgBoxResult.Yes Then
         End
      End If

   End Sub

   Private Sub btnInfo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnInfo.Click
      frmInfo.Show()
   End Sub

   Private Sub Noty_MouseDoubleClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Noty.MouseDoubleClick
      Me.Show()
      Me.ShowInTaskbar = True
   End Sub

#End Region



   

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Dim portName As String
        portName = InputBox("Enter Port", , "COM6")
        COM = New System.IO.Ports.SerialPort(portName)
    End Sub
End Class
