/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

/**
 * @file		ScatterWeb.Loader.c
 * @ingroup		loader
 * @brief		Bootstrap loader
 * @author		Michael Baar	<baar@inf.fu-berlin.de>

This bootstrap loader can be flashed on a node (alone or together with an
application). It can run single time or during every boot to check and perform
application updates. After completion it starts the application.

The loader includes the sd-card driver and optionally FAT16 support (set
LOADER_WITH_FAT in AppConf.h).

Applications that shall run on a node with bootloader need to be linked using
the ldbldapp.x linker-script.
 */

#include "app.h"

sd_cache_t 				buffer;

/**
 * @name Private interface
 */
void wait(unsigned long n);
void visualMessage(unsigned char n);
//@}

/**
 * @name Memory alignment
 * @{
 */
#ifdef __TI_COMPILER_VERSION__
	/*
	#pragma DATA_SECTION(__stack, ".stack")
	int  __stack;
	void main();
	#pragma DATA_SECTION(_reset_vector__,".reset")
	void (* const _reset_vector__)() = &main;
	#pragma FUNC_NEVER_RETURNS(main)
	*/
#else
	extern int  __stack;
	void loader() __attribute__ ((naked));
	__attribute__((section(".reset")))	
	void (* const _reset_vector__)() = &main;
#endif			
///@}

void main() {
		int test = 0;
	
	//
	// init system
	//
	_DINT();
	DMA0CTL = 0;						// hold DMA
	WDTCTL = WDTPW | WDTHOLD;			// hold WD
	IFG1=0;             // clear interrupt flag1 
	
						// DCO(SMCLK) is 2,4576MHz, /6 = 409600 Hz 
	FCTL2=0xA5C5;       // select SMCLK for flash timing, divider 5+1 
	
	IE1=0x00;           // disable all NMI-Interrupt sources
	IE2=0x00;

	#ifdef __TI_COMPILER_VERSION__
		//asm("\t MOV.W\t #__stack,SP");	// set stackpointer
	#else
		WRITE_SP(&__stack);
	#endif
	
	// configure LED
	P5SEL &= ~0x80;
	P5DIR |= 0x80;

	// configure UART
	System_setDCO();
	uart_init();
	#if LOADER_WITH_PRINTF
		uart_set_speed( UART_MODE_RS232, 0x27, 0x00, 0x10 );
		uart_lock(UART_MODE_RS232);
		uart_unlock(UART_MODE_RS232);
		_DINT();
		printf("ScatterWeb Bootloader in Debugmode\n");
	#endif	
	libSD_Init();
	
	//
	// init file access
	//
	if( test != 0 ) {
	if( Sd_init_card(0) != SD_INIT_SUCCESS )
		goto nomedia;
	}
	visualMessage(1);

nomedia:
	visualMessage(8);
	LED_ON;
    WDTCTL = WDTPW + WDTHOLD;
    LPM1;
    goto nomedia;
}

void wait(unsigned long n) {
	volatile unsigned long i;
	for( i = 0; i < n; i++ ) { _NOP(); }
}
void visualMessage(unsigned char n) {
	unsigned int j, k;
	
	LED_OFF;
	for( k = 0; k < n; k++ ) {
		wait(80000);
		for( j = 0; j < 3; j++ ) {
			LED_ON;
			wait(10000);
			LED_OFF;
			wait(20000);
		}
	}
	wait(80000);
}

